from .S2Identified import S2Identified
from .S2SequenceAnnotation import S2SequenceAnnotation
from .S2Sequence import S2Sequence
from .S2SequenceConstraint import S2SequenceConstraint
from .S2IdentifiedFactory import S2IdentifiedFactory
from .S2MapsTo import S2MapsTo

from .terms import SBOL2

from rdflib import URIRef
from rdflib.namespace import RDF


class S2ComponentDefinition(S2Identified):
    def __init__(self, g, uri):
        super(S2ComponentDefinition, self).__init__(g, uri)

    @property
    def types(self):
        return self.get_uri_properties(SBOL2.type)

    def has_type(self, the_type):
        return self.g.hasMatch(self.uri, SBOL2.type, URIRef(the_type))

    def add_type(self, the_type):
        self.insert_properties({SBOL2.type: URIRef(the_type)})

    @property
    def roles(self):
        return self.get_uri_properties(SBOL2.role)

    def has_role(self, role):
        return self.g.hasMatch(self.uri, SBOL2.role, URIRef(role))

    def add_role(self, role):
        self.insert_properties({SBOL2.role: URIRef(role)})

    @property
    def components(self):
        return [S2Component(self.g, uri) for uri in self.get_uri_properties(SBOL2.component)]

    def create_component(self, display_id, definition):
        identified = S2IdentifiedFactory.create_child(self.g, SBOL2.Component, self, display_id)
        new_component = S2Component(self.g, identified.uri)
        new_component.definition = definition
        new_component.access = SBOL2.public
        self.insert_uri_property(SBOL2.component, new_component.uri)
        return new_component

    @property
    def sequence_annotations(self):
        return [S2SequenceAnnotation(self.g, uri) for uri in self.get_uri_properties(SBOL2.sequenceAnnotation)]

    @property
    def sequences(self):
        return [S2Sequence(self.g, uri) for uri in self.get_uri_properties(SBOL2.sequence)]

    def add_sequence(self, sequence):
        self.insert_identified_property(SBOL2.sequence, sequence)

    def create_sequence_constraint(self, display_id, restriction, a, b):
        identified = S2IdentifiedFactory.create_child(self.g, SBOL2.SequenceConstraint, self, display_id)
        sc = S2SequenceConstraint(self.g, identified.uri)
        sc.subject = a
        sc.object = b
        sc.restriction = restriction
        self.insert_uri_property(SBOL2.sequenceConstraint, sc.uri)
        return sc

    @property
    def instances(self):
        triples = list(self.g.triples((None, SBOL2.definition, URIRef(self.uri))))
        return [S2Component(self.g, triple[0].toPython()) for triple in triples]

class S2Component(S2Identified):
    def __init__(self, g, uri):
        super(S2Component, self).__init__(g, uri)

    @property
    def definition(self):
        return S2ComponentDefinition(self.g, self.get_uri_property(SBOL2.definition))

    @definition.setter
    def definition(self, definition):
        self.set_uri_property(SBOL2.definition, definition.uri)

    def create_maps_to(self, display_id, refinement, local, remote):
        identified = S2IdentifiedFactory.create_child(self.g, SBOL2.MapsTo, self, display_id)
        maps_to = S2MapsTo(self.g, identified.uri)
        maps_to.local = local
        maps_to.remote = remote
        maps_to.refinement = refinement
        self.insert_uri_property(SBOL2.mapsTo, maps_to.uri)
        return maps_to

    @property
    def access(self):
        return self.get_uri_property(SBOL2.access)

    @access.setter
    def access(self, access):
        self.set_uri_property(SBOL2.access, access)

    @property
    def containing_component_definition(self):
        triples = list(self.g.triples((None, SBOL2.component, URIRef(self.uri))))
        for triple in triples:
            if (triple[0], RDF.type, SBOL2.ComponentDefinition) in self.g.g:
                return S2ComponentDefinition(self.g, triple[0].toPython())
        raise Exception('component ' + self.uri + ' not contained by a definition?')
        
    @property
    def measure(self):
        return self.get_identified_property(SBOL2.measure)

    @measure.setter
    def measure(self, measure):
        self.set_identified_property(SBOL2.measure, measure)





from .S2Identified import S2Identified

from .terms import SBOL2

from rdflib import URIRef
from rdflib.namespace import RDF

class S2Attachment(S2Identified):
    def __init__(self, g, uri):
        super(S2Attachment, self).__init__(g, uri)

    @property
    def source(self):
        return self.get_uri_property(SBOL2.source)

    @source.setter
    def source(self, source):
        self.set_uri_property(SBOL2.source, source)

    @property
    def format(self):
        return self.get_uri_property(SBOL2.format)

    @format.setter
    def format(self, format):
        self.set_uri_property(SBOL2.format, format)

    @property
    def size(self):
        return self.get_integer_property(SBOL2.size)

    @size.setter
    def size(self, size):
        self.set_integer_property(SBOL2.size, size)

    @property
    def hash(self):
        return self.get_string_property(SBOL2.hash)

    @hash.setter
    def hash(self, hash):
        self.set_string_property(SBOL2.hash, hash)




from .S2Identified import S2Identified

from .terms import SBOL2
from .terms import Measure

class S2Measure(S2Identified):
    def __init__(self, g, uri):
        super(S2Measure, self).__init__(g, uri)

    @property
    def value(self):
        return self.get_double_property(Measure.hasNumericalValue)

    @value.setter
    def value(self, value):
        self.set_double_property(Measure.hasNumericalValue, value)

    @property
    def unit(self):
        return self.get_identified_property(Measure.hasUnit)

    @unit.setter
    def unit(self, unit):
        self.set_identified_property(Measure.hasUnit, unit)




from .S2Identified import S2Identified

from .terms import SBOL2


class S2Sequence(S2Identified):
    def __init__(self, g, uri):
        super(S2Sequence, self).__init__(g, uri)

    @property
    def encoding(self):
        return self.get_uri_property(SBOL2.encoding)

    @encoding.setter
    def encoding(self, encoding):
        self.set_uri_property(SBOL2.encoding, encoding)

    @property
    def elements(self):
        return self.get_string_property(SBOL2.elements)

    @elements.setter
    def elements(self, elements):
        self.set_string_property(SBOL2.elements, elements)



from .S2Location import S2Location

from .terms import SBOL2


class S2Range(S2Location):
    def __init__(self, g, uri):
        super(S2Range, self).__init__(g, uri)

    @property
    def start(self):
        return self.get_integer_property(SBOL2.start)

    @property
    def end(self):
        return self.get_integer_property(SBOL2.end)

    @property
    def orientation(self):
        return self.get_uri_property(SBOL2.orientation)



from rdflib.namespace import ClosedNamespace
from rdflib import URIRef

Dcterms = ClosedNamespace(
    uri=URIRef("http://purl.org/dc/terms/"),
    terms=[
        "title"
    ]
)

SBOL2 = ClosedNamespace(
    uri=URIRef("http://sbols.org/v2#"),
    terms=[
        # types
        "ComponentDefinition",
        "Component",
        "ModuleDefinition",
        "Module",
        "Interaction",
        "Participation",
        "Model",
        "MapsTo",
        "Sequence",
        "SequenceConstraint",
        "FunctionalComponent",
        "Experiment",
        "ExperimentalData",

        # predicates
        "component",
        "displayId",
        "type",
        "role",
        "persistentIdentity",
        "version",
        "interaction",
        "participant",
        "mapsTo",
        "sequence",
        "access",
        "definition",
        "direction",
        "refinement",
        "sequenceConstraint",
        "functionalComponent",
        "module",
        "participation",
        "encoding",
        "elements",
        "restriction",
        "model",
        "framework",
        "source",
        "language",
        "attachment",
        "format",
        "hash",
        "size",
        "built",
        "experimentalData",
        "sourceLocation",

        # terms
        "precedes",
        "local",
        "remote",
        "private",
        "public",
        "none",
        "useLocal",
        "useRemote",
        "merge",
        "verifyIdentical",
        "design",
        "build",
        "test",
        "learn"
    ]
)

Biopax = ClosedNamespace(
    uri=URIRef("http://www.biopax.org/release/biopax-level3.owl#"),
    terms=[
        "DnaRegion",
        "RnaRegion",
        "Protein",
        "Complex",
        "SmallMolecule"
    ]
)

Prov = ClosedNamespace(
    uri=URIRef("http://www.w3.org/ns/prov#"),
    terms=[

        # types
        "Agent",
        "Association",
        "Activity",
        "Usage",
        "Plan",
        "Usage",

        # predicates
        "wasDerivedFrom",
        "wasGeneratedBy",
        "qualifiedAssociation",
        "qualifiedUsage",
        "startedAtTime",
        "endedAtTime",
        "wasInformedBy",
        "agent",
        "entity",
        "hadRole",
        "hadPlan"
    ]
)

Measure = ClosedNamespace(
    uri=URIRef("http://www.ontology-of-units-of-measure.org/resource/om-2/"),
    terms=[

        # types
        "Measure",

        # predicates
        "hasNumericalValue",
        "hasUnit"
    ]
)



from .S2Identified import S2Identified

class S2Model(S2Identified):
    def __init__(self, g, uri):
        super(S2Model, self).__init__(g, uri)

    @property
    def framework(self):
        return self.get_uri_property(SBOL2.framework)

    @framework.setter
    def framework(self, framework):
        self.set_uri_property(SBOL2.framework, framework)

    @property
    def language(self):
        return self.get_uri_property(SBOL2.language)

    @language.setter
    def language(self, language):
        self.set_uri_property(SBOL2.language, language)

    @property
    def source(self):
        return self.get_uri_property(SBOL2.source)

    @source.setter
    def source(self, source):
        self.set_uri_property(SBOL2.source, source)



from .S2Identified import S2Identified

from .terms import SBOL2


class S2SequenceConstraint(S2Identified):
    def __init__(self, g, uri):
        super(S2SequenceConstraint, self).__init__(g, uri)

    @property
    def restriction(self):
        return self.get_uri_property(SBOL2.restriction)

    @restriction.setter
    def restriction(self, restriction):
        self.set_uri_property(SBOL2.restriction, restriction)



from .S2Identified import S2Identified
from .S2IdentifiedFactory import S2IdentifiedFactory
from .S2Interaction import S2Interaction
from .S2FunctionalComponent import S2FunctionalComponent
from .S2MapsTo import S2MapsTo
from .S2Component import S2ComponentDefinition

from .terms import SBOL2

from rdflib import URIRef


class S2ModuleDefinition(S2Identified):
    def __init__(self, g, uri):
        super(S2ModuleDefinition, self).__init__(g, uri)

    @property
    def roles(self):
        return self.get_uri_properties(SBOL2.role)

    def has_role(self, role):
        return self.g.hasMatch(self.uri, SBOL2.role, URIRef(role))

    def add_role(self, role):
        self.insert_properties({SBOL2.role: URIRef(role)})

    @property
    def functional_components(self):
        return [S2FunctionalComponent(self.g, uri) for uri in self.get_uri_properties(SBOL2.functionalComponent)]

    @property
    def interactions(self):
        return [S2Interaction(self.g, uri) for uri in self.get_uri_properties(SBOL2.interaction)]

    @property
    def modules(self):
        return [S2Module(self.g, uri) for uri in self.get_uri_properties(SBOL2.module)]

    def create_interaction(self, display_id, the_type):
        print('ci params are: %s, %s, %s' %(self, display_id, the_type))
        print('self pi and version are %s, %s' % (self.persistent_identity, self.version))
        identified = S2IdentifiedFactory.create_child(self.g, SBOL2.Interaction, self, display_id)
        interaction = S2Interaction(self.g, identified.uri)
        interaction.type = the_type
        self.insert_uri_property(SBOL2.interaction, interaction.uri)
        return interaction

    def create_functional_component(self, display_id, access, definition, direction):
        identified = S2IdentifiedFactory.create_child(self.g, SBOL2.FunctionalComponent, self, display_id)
        fc = S2FunctionalComponent(self.g, identified.uri)
        fc.access = access
        fc.definition = definition
        fc.direction = direction
        self.insert_uri_property(SBOL2.functionalComponent, fc.uri)
        return fc

    def create_module(self, display_id, definition):
        identified = S2IdentifiedFactory.create_child(self.g, SBOL2.Module, self, display_id)
        new_module = S2Module(self.g, identified.uri)
        new_module.definition = definition
        self.insert_uri_property(SBOL2.module, new_module.uri)
        return new_module


class S2Module(S2Identified):
    def __init__(self, g, uri):
        super(S2Module, self).__init__(g, uri)

    @property
    def definition(self):
        return S2ComponentDefinition(self.g, self.get_uri_property(SBOL2.definition))

    @definition.setter
    def definition(self, definition):
        self.set_identified_property(SBOL2.definition, definition)

    def create_maps_to(self, display_id, refinement, local, remote):
        identified = S2IdentifiedFactory.create_child(self.g, SBOL2.MapsTo, self, display_id)
        maps_to = S2MapsTo(self.g, identified.uri)
        maps_to.local = local
        maps_to.remote = remote
        maps_to.refinement = refinement
        self.insert_uri_property(SBOL2.mapsTo, maps_to.uri)
        return maps_to




from .S2Identified import S2Identified

from .terms import Prov
from .terms import SBOL2

from rdflib import URIRef
from rdflib.namespace import RDF

class S2Experiment(S2Identified):
    def __init__(self, g, uri):
        super(S2Experiment, self).__init__(g, uri)

    @property
    def experimental_data(self):
        return [self.g.uri_to_facade(uri) for uri in self.get_uri_properties(SBOL2.experimentalData)]

    def add_experimental_data(self, ed):
        self.insert_identified_property(SBOL2.experimentalData, ed)





from rdflib import Literal, URIRef
from rdflib.namespace import RDF
from .terms import SBOL2
from .S2Identified import S2Identified
from .terms import Dcterms

from . import CompliantURIs


class S2IdentifiedFactory:
    def __init__(self):
        pass

    @staticmethod
    def create_top_level(graph, the_type, uri_prefix, display_id, name=None, version="1"):

        uri = graph.generate_uri(uri_prefix + display_id + '$n?$/' + version)

        graph.insert_properties(uri, {
            RDF.type: URIRef(the_type),
            SBOL2.displayId: Literal(CompliantURIs.get_display_id(uri)),
            SBOL2.persistentIdentity: URIRef(CompliantURIs.get_persistent_identity(uri)),
            SBOL2.version: Literal(CompliantURIs.get_version(uri))
        })

        if name:
            graph.insert_properties(uri, {Dcterms.title: Literal(name)})

        return S2Identified(graph, uri)

    @staticmethod
    def create_child(graph, the_type, parent, display_id, name=None):

        version = parent.version

        print("%s %s %s" % (parent.persistent_identity, display_id, version))

        uri = graph.generate_uri(parent.persistent_identity + '/' + display_id + '$n?$/' + version)

        graph.insert_properties(uri, {
            RDF.type: URIRef(the_type),
            SBOL2.displayId: Literal(CompliantURIs.get_display_id(uri)),
            SBOL2.persistentIdentity: URIRef(CompliantURIs.get_persistent_identity(uri)),
            SBOL2.version: Literal(CompliantURIs.get_version(uri))
        })

        if name is not None:
            graph.insert_properties(uri, {Dcterms.title: Literal(name)})

        return S2Identified(graph, uri)



from .S2Location import S2Location

from .terms import SBOL2


class S2GenericLocation(S2Location):
    def __init__(self, g, uri):
        super(S2GenericLocation, self).__init__(g, uri)

    @property
    def orientation(self):
        return self.get_uri_property(SBOL2.orientation)




from .S2Identified import S2Identified

from .terms import Prov
from .terms import SBOL2

from rdflib import URIRef
from rdflib.namespace import RDF

class S2ProvPlan(S2Identified):
    def __init__(self, g, uri):
        super(S2ProvPlan, self).__init__(g, uri)





from .S2Identified import S2Identified

from .terms import SBOL2

from rdflib import URIRef


class S2Participation(S2Identified):
    def __init__(self, g, uri):
        super(S2Participation, self).__init__(g, uri)

    @property
    def roles(self):
        return self.get_uri_properties(SBOL2.role)

    def has_role(self, role):
        return self.g.hasMatch(self.uri, SBOL2.role, URIRef(role))

    def add_role(self, role):
        self.insert_identified_property(SBOL2.role, URIRef(role))

    @property
    def participant(self):
        return self.get_identified_property(SBOL2.participant)

    @participant.setter
    def participant(self, participant):
        self.set_identified_property(SBOL2.participant, participant)

    @property
    def measure(self):
        return self.get_identified_property(SBOL2.measure)

    @measure.setter
    def measure(self, measure):
        self.set_identified_property(SBOL2.measure, measure)



from .S2Identified import S2Identified
from .S2Participation import S2Participation
from .S2IdentifiedFactory import S2IdentifiedFactory

from .terms import SBOL2


class S2Interaction(S2Identified):
    def __init__(self, g, uri):
        super(S2Interaction, self).__init__(g, uri)

    @property
    def type(self):
        return self.get_uri_property(SBOL2.type)

    @type.setter
    def type(self, the_type):
        self.set_uri_property(SBOL2.type, the_type)

    @property
    def participations(self):
        return [S2Participation(self.g, uri) for uri in self.get_uri_properties(SBOL2.participation)]

    def create_participation(self, display_id, participant, role):
        identified = S2IdentifiedFactory.create_child(self.g, SBOL2.Participation, self, display_id)
        participation = S2Participation(self.g, identified.uri)
        participation.participant = participant
        participation.add_role(role)
        self.insert_uri_property(SBOL2.participation, participation.uri)
        return participation

    @property
    def measure(self):
        return self.get_identified_property(SBOL2.measure)

    @measure.setter
    def measure(self, measure):
        self.set_identified_property(SBOL2.measure, measure)

    


from rdflib import Literal, URIRef

from .terms import SBOL2, Dcterms


class Facade(object):
    def __init__(self, g, uri):
        self.g = g
        self.uri = uri

    def get_string_property(self, predicate):
        for triple in self.g.triples((URIRef(self.uri), URIRef(predicate), None)):
            return triple[2].toPython()
        return None

    def set_string_property(self, predicate, value):
        self.g.remove((URIRef(self.uri), URIRef(predicate), None))
        self.g.add((URIRef(self.uri), URIRef(predicate), Literal(value)))

    def get_integer_property(self, predicate):
        for triple in self.g.triples((URIRef(self.uri), URIRef(predicate), None)):
            return triple[2].toPython()
        return None

    def set_integer_property(self, predicate, value):
        self.g.remove((URIRef(self.uri), URIRef(predicate), None))
        self.g.add((URIRef(self.uri), URIRef(predicate), Literal(value)))

    def get_double_property(self, predicate):
        for triple in self.g.triples((URIRef(self.uri), URIRef(predicate), None)):
            return triple[2].toPython()
        return None

    def set_double_property(self, predicate, value):
        self.g.remove((URIRef(self.uri), URIRef(predicate), None))
        self.g.add((URIRef(self.uri), URIRef(predicate), Literal(value)))

    def set_uri_property(self, predicate, value):
        self.g.remove((URIRef(self.uri), URIRef(predicate), None))
        self.g.add((URIRef(self.uri), URIRef(predicate), URIRef(value)))

    def insert_uri_property(self, predicate, value):
        self.g.add((URIRef(self.uri), URIRef(predicate), URIRef(value)))

    def get_uri_property(self, predicate):
        for triple in self.g.triples((URIRef(self.uri), URIRef(predicate), None)):
            return triple[2].toPython()
        return None

    def get_uri_properties(self, predicate):
        return [triple[2].toPython() for triple in self.g.triples((URIRef(self.uri), URIRef(predicate), None))]

    def insert_properties(self, properties):
        for predicate in properties:
            obj = properties[predicate]
            self.g.add((URIRef(self.uri), URIRef(predicate), obj))

    def set_identified_property(self, predicate, value):
        if isinstance(value, S2Identified):
            self.set_uri_property(URIRef(predicate), URIRef(value.uri))
        elif isinstance(value, str):
            self.set_uri_property(URIRef(predicate), URIRef(value))
        else:
            raise Exception()

    def get_identified_property(self, predicate):
        return self.g.uri_to_facade(self.get_uri_property(predicate))

    def insert_identified_property(self, predicate, value):
        if isinstance(value, S2Identified):
            self.insert_uri_property(URIRef(predicate), URIRef(value.uri))
        elif isinstance(value, URIRef):
            self.insert_uri_property(URIRef(predicate), value)
        elif isinstance(value, str):
            self.insert_uri_property(URIRef(predicate), URIRef(value))
        else:
            print('you asked me to insert %s but i do not know how' % value)
            raise Exception()


class S2Identified(Facade):
    def __init__(self, g, uri):
        super(S2Identified, self).__init__(g, uri)

    @property
    def name(self):
        return self.get_string_property(Dcterms.title)

    @name.setter
    def name(self, name):
        self.set_string_property(Dcterms.title, name)

    @property
    def display_id(self):
        return self.get_string_property(SBOL2.displayId)

    @property
    def persistent_identity(self):
        return self.get_uri_property(SBOL2.persistentIdentity)

    @property
    def version(self):
        return self.get_uri_property(SBOL2.version)

    @property
    def display_name(self):
        name = self.name
        if name is not None:
            return name
        return self.display_id




from .S2Identified import S2Identified

from .terms import Prov
from .terms import SBOL2

from rdflib import URIRef
from rdflib.namespace import RDF

class S2ProvActivity(S2Identified):
    def __init__(self, g, uri):
        super(S2ProvActivity, self).__init__(g, uri)

    @property
    def types(self):
        return self.get_uri_properties(SBOL2.type)

    def has_type(self, the_type):
        return self.g.hasMatch(self.uri, SBOL2.type, URIRef(the_type))

    def add_type(self, the_type):
        self.insert_properties({SBOL2.type: URIRef(the_type)})

    @property
    def plan(self):
        return self.get_identified_property(Prov.hadPlan)

    @plan.setter
    def plan(self, plan):
        self.set_identified_property(Prov.hadPlan, plan)

    @property
    def usages(self):
        return [self.g.uri_to_facade(uri) for uri in self.get_uri_properties(Prov.qualifiedUsage)]

    def add_usage(self, usage):
        self.insert_identified_property(Prov.qualifiedUsage, usage)

    @property
    def associations(self):
        return [self.g.uri_to_facade(uri) for uri in self.get_uri_properties(Prov.qualifiedAssociation)]

    def add_association(self, association):
        self.insert_identified_property(Prov.qualifiedAssociation, association)






from .S2Identified import S2Identified

from .terms import Prov
from .terms import SBOL2

from rdflib import URIRef
from rdflib.namespace import RDF

class S2ProvAssociation(S2Identified):
    def __init__(self, g, uri):
        super(S2ProvAssociation, self).__init__(g, uri)

    @property
    def agent(self):
        return self.get_identified_property(Prov.agent)

    @agent.setter
    def agent(self, agent):
        self.set_identified_property(Prov.agent, agent)

    @property
    def plan(self):
        return self.get_identified_property(Prov.hadPlan)

    @plan.setter
    def plan(self, plan):
        self.set_identified_property(Prov.hadPlan, plan)

    @property
    def role(self):
        return self.get_uri_property(Prov.hadRole)

    @role.setter
    def role(self, role):
        self.set_uri_property(Prov.hadRole, role)



import rdflib
import requests
import json

from rdflib import URIRef

from rdflib.namespace import RDF

from .terms import SBOL2
from .terms import Prov
from .terms import Measure

from .S2Component import S2Component, S2ComponentDefinition
from .S2Module import S2Module, S2ModuleDefinition
from .S2Sequence import S2Sequence

from .S2IdentifiedFactory import S2IdentifiedFactory
from .S2SequenceAnnotation import S2SequenceAnnotation
from .S2SequenceConstraint import S2SequenceConstraint
from .S2Model import S2Model
from .S2Range import S2Range
from .S2Cut import S2Cut
from .S2GenericLocation import S2GenericLocation
from .S2Experiment import S2Experiment
from .S2ExperimentalData import S2ExperimentalData
from .S2ProvActivity import S2ProvActivity
from .S2ProvAgent import S2ProvAgent
from .S2ProvAssociation import S2ProvAssociation
from .S2ProvPlan import S2ProvPlan
from .S2ProvUsage import S2ProvUsage
from .S2Measure import S2Measure

from .SBOL2Serialize import serialize_sboll2


class SBOL2Graph(rdflib.Graph):
    def __init__(self):
        super(SBOL2Graph, self).__init__()

    @property
    def component_definitions(self):
        return [S2ComponentDefinition(self, triple[0]) for triple in
                self.triples((None, RDF.type, SBOL2.ComponentDefinition))]

    @property
    def module_definitions(self):
        return [S2ModuleDefinition(self, triple[0]) for triple in
                self.triples((None, RDF.type, SBOL2.ModuleDefinition))]

    @property
    def sequences(self):
        return [S2Sequence(self, triple[0]) for triple in
                self.triples((None, RDF.type, SBOL2.Sequence))]

    def get_type(self, uri):
        triples = self.triples(uri, RDF.type, None)
        if len(triples) > 0:
            return triples[0][2].toPython()
        else:
            return None

    def create_component_definition(self, uri_prefix, display_id, the_type, version="1"):
        identified = S2IdentifiedFactory.create_top_level(self, SBOL2.ComponentDefinition, uri_prefix, display_id, None,
                                                          version)
        cd = S2ComponentDefinition(self, identified.uri)
        cd.add_type(the_type)
        return cd

    def create_module_definition(self, uri_prefix, display_id, version="1"):
        identified = S2IdentifiedFactory.create_top_level(self, SBOL2.ModuleDefinition, uri_prefix, display_id, None,
                                                          version)
        md = S2ModuleDefinition(self, identified.uri)
        return md

    def create_sequence(self, uri_prefix, display_id, elements, encoding, version="1"):
        identified = S2IdentifiedFactory.create_top_level(self, SBOL2.Sequence, uri_prefix, display_id, None, version)
        seq = S2Sequence(self, identified.uri)
        seq.encoding = encoding
        seq.elements = elements
        return seq

    def generate_uri(self, template):

        n = 1

        while True:

            if n > 1:
                foobar = '_' + str(n)
            else:
                foobar = ''

            # uri = template.replace('$rand$', shortid.generate()).replace('$^n$', '' + n).replace('$n$', '_' + n).replace('$n?$', foobar)
            uri = template.replace('$^n$', str(n)).replace('$n$', '_' + str(n)).replace('$n?$', foobar)

            n = n + 1

            # TODO!!!!
            if len(list(self.triples((uri, None, None)))) > 0:
                continue

            return uri

    def insert_properties(self, uri, properties):
        for predicate in properties:
            obj = properties[predicate]
            self.add((URIRef(uri), URIRef(predicate), obj))

    def uri_to_facade(self, uri):
        the_type = self.get_type(uri)
        if the_type is None:
            return None
        if the_type == SBOL2.ComponentDefinition:
            return S2ComponentDefinition(self, uri)
        if the_type == SBOL2.Component:
            return S2Component(self, uri)
        if the_type == SBOL2.SequenceAnnotation:
            return S2SequenceAnnotation(self, uri)
        if the_type == SBOL2.SequenceConstraint:
            return S2SequenceConstraint(self, uri)
        if the_type == SBOL2.ModuleDefinition:
            return S2ModuleDefinition(self, uri)
        if the_type == SBOL2.Module:
            return S2Module(self, uri)
        if the_type == SBOL2.Sequence:
            return S2Sequence(self, uri)
        if the_type == SBOL2.Model:
            return S2Model(self, uri)
        if the_type == SBOL2.Range:
            return S2Range(self, uri)
        if the_type == SBOL2.Cut:
            return S2Cut(self, uri)
        if the_type == SBOL2.GenericLocation:
            return S2GenericLocation(self, uri)
        if the_type == SBOL2.Experiment:
            return S2Experiment(self, uri)
        if the_type == SBOL2.ExperimentalData:
            return S2ExperimentalData(self, uri)
        if the_type == Prov.Activity:
            return S2ProvActivity(self, uri)
        if the_type == Prov.Agent:
            return S2ProvAgent(self, uri)
        if the_type == Prov.Association:
            return S2ProvAssociation(self, uri)
        if the_type == Prov.Plan:
            return S2ProvPlan(self, uri)
        if the_type == Prov.Usage:
            return S2ProvUsage(self, uri)
        if the_type == Measure.Measure:
            return S2Measure(self, uri)
        return None

    def serialize_xml(self):
        return serialize_sboll2(self)

    @staticmethod
    def validate_xml(xml, check_uri_compliance=False, check_completeness=False, check_best_practices=False,
                     provide_detailed_stack_trace=False):

        # See API docs at http://synbiodex.github.io/SBOL-Validator/#introduction
        request = {'options': {'language': 'SBOL2',
                               'test_equality': False,
                               'check_uri_compliance': check_uri_compliance,
                               'check_completeness': check_completeness,
                               'check_best_practices': check_best_practices,
                               'fail_on_first_error': False,
                               'provide_detailed_stack_trace': provide_detailed_stack_trace,
                               'subset_uri': '',
                               'uri_prefix': '',
                               'version': '',
                               'insert_type': False,
                               'main_file_name': 'main file',
                               'diff_file_name': 'comparison file',
                               },
                   'return_file': True,
                   'main_file': xml
                   }

        resp = requests.post("http://www.async.ece.utah.edu/validate/", json=request)
        return json.dumps(resp.json())



from .S2Identified import S2Identified
from .S2Component import S2ComponentDefinition
from .S2IdentifiedFactory import S2IdentifiedFactory
from .S2MapsTo import S2MapsTo

from .terms import SBOL2


class S2FunctionalComponent(S2Identified):
    def __init__(self, g, uri):
        super(S2FunctionalComponent, self).__init__(g, uri)

    @property
    def definition(self):
        return S2ComponentDefinition(self.g, self.get_uri_property(SBOL2.definition))

    @definition.setter
    def definition(self, definition):
        self.set_identified_property(SBOL2.definition, definition)

    @property
    def access(self):
        return self.get_uri_property(SBOL2.access)

    @access.setter
    def access(self, access):
        self.set_uri_property(SBOL2.access, access)

    @property
    def direction(self):
        return self.get_uri_property(SBOL2.direction)

    @direction.setter
    def direction(self, direction):
        self.set_uri_property(SBOL2.direction, direction)

    def create_maps_to(self, display_id, refinement, local, remote):
        identified = S2IdentifiedFactory.create_child(self.g, SBOL2.MapsTo, self, display_id)
        maps_to = S2MapsTo(self.g, identified.uri)
        maps_to.local = local
        maps_to.remote = remote
        maps_to.refinement = refinement
        self.insert_uri_property(SBOL2.mapsTo, maps_to.uri)
        return maps_to

    @property
    def measure(self):
        return self.get_identified_property(SBOL2.measure)

    @measure.setter
    def measure(self, measure):
        self.set_identified_property(SBOL2.measure, measure)

    


def get_display_id(uri):
    tokens = uri.split('/')
    return tokens[-2]


def get_version(uri):
    tokens = uri.split('/')
    return tokens[-1]


def get_persistent_identity(uri):
    tokens = uri.split('/')
    return '/'.join(tokens[:-1])



from .S2Identified import S2Identified

from .terms import SBOL2
from rdflib import URIRef


class S2SequenceAnnotation(S2Identified):
    def __init__(self, g, uri):
        super(S2SequenceAnnotation, self).__init__(g, uri)

    @property
    def roles(self):
        return self.get_uri_properties(SBOL2.role)

    def has_role(self, role):
        return self.g.hasMatch(self.uri, SBOL2.role, URIRef(role))

    def add_role(self, role):
        self.insert_identified_property(SBOL2.role, URIRef(role))

    @property
    def locations(self):
        return [self.g.uri_to_facade(uri) for uri in self.get_uri_properties(SBOL2.location)]

    def add_location(self, location):
        self.insert_identified_property(SBOL2.location, location)



from distutils.core import setup
import setuptools  # noqa

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(name='pysbolgraph',
      version='0.2.0',
      description='A simple Python library to read and write SBOL files',
      long_description=long_description,
      long_description_content_type="text/markdown",
      author='James Alastair McLaughlin',
      author_email='james@mclgh.net',
      url='https://github.com/udp/pysbolgraph',
      packages=['pysbolgraph'],
      install_requires=['rdflib', 'lxml', 'requests'],
      classifiers=[
          "Programming Language :: Python :: 3",
          "Programming Language :: Python :: 2",
          "License :: OSI Approved :: BSD License",
          "Operating System :: OS Independent",
      ]
      )




from .S2Identified import S2Identified

from .terms import SBOL2

class S2Location(S2Identified):
    def __init__(self, g, uri):
        super(S2Location, self).__init__(g, uri)

    @property
    def sequence(self):
        return self.get_identified_property(SBOL2.sequence)

    @sequence.setter
    def sequence(self, sequence):
        self.set_identified_property(SBOL2.sequence, sequence)

    


from .S2Identified import S2Identified

from .terms import SBOL2


class S2MapsTo(S2Identified):
    def __init__(self, g, uri):
        super(S2MapsTo, self).__init__(g, uri)

    @property
    def local(self):
        return self.g.uri_to_facade(self.get_uri_property(SBOL2.local))

    @local.setter
    def local(self, local):
        self.set_identified_property(SBOL2.local, local)

    @property
    def remote(self):
        return self.g.uri_to_facade(self.get_uri_property(SBOL2.remote))

    @remote.setter
    def remote(self, remote):
        self.set_identified_property(SBOL2.remote, remote)

    @property
    def refinement(self):
        return self.get_uri_property(SBOL2.refinement)

    @refinement.setter
    def refinement(self, refinement):
        self.set_uri_property(SBOL2.refinement, refinement)
